*************************************************************************************
*														*
*	This is a Frankenstein like combination of a 3D environment display and		*
*	a swarm simulation. Double buffering is used to render red wasps chased		*
*	by swarms of green bees all in a 3D cubic bounding box.	2007/04/18		*
*														*
*	While this all looks nice there is a shortcomming, no depth sorting is done	*
*	so nearer objects may be obscured by more distant objects depending on the	*
*	order in which they are drawn.								*
*														*
*	Use the cursor keys to rotate the environment and move it up and down and	*
*	use the page up and page down keys to move the environment away or closer.	*
*														*
*	Use [S] to toggle the status display on/off and the following key pairs ..	*
*														*
*		[B] plus [currt] to increase the bee maximum velocity				*
*		[B] plus [curlt] to decrease the bee maximum velocity				*
*		[B] plus [curup] to increase the bee maximum acceleration			*
*		[B] plus [curdn] to decrease the bee maximum acceleration			*
*														*
*		[W] plus [currt] to increase the wasp maximum velocity			*
*		[W] plus [curlt] to decrease the wasp maximum velocity			*
*		[W] plus [curup] to increase the wasp maximum acceleration			*
*		[W] plus [curdn] to decrease the wasp maximum acceleration			*
*														*
*	For bees that don't stick so tightly to the wasps increase the bee maximum	*
*	velocity , for wasps that get away more increase the wasp maximum velocity	*
*	and wasp_acceleration.										*
*														*
*	The F2, F3 and F4	keys can be used to select a screen size of 640 x 480,	*
*	800 x 600 and 1024 x 768 respectively.							*
*														*
*	This version for Sim68K 3.6 or later							*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	 http://members.lycos.co.uk/leeedavison/index.html					*
*														*
*	mail : leeedavison@lycos.co.uk								*
*														*
*************************************************************************************
*
* constants

number_of_wasps	EQU	4			* number of wasps
number_of_bees	EQU	20			* number of bees

* for bees that don't stick so tightly to the wasps increase bee_velocity_max
* for wasps that get away more increase wasp_velocity_max and wasp_acceleration

acceleration_max	EQU	10			* maximum wasp or bee acceleration
acceleration_min	EQU	2			* minimum wasp or bee acceleration
acceleration_stp	EQU	1			* wasp or bee acceleration step size

velocity_max	EQU	500			* maximum wasp or bee velocity^2
velocity_min	EQU	50			* minimum wasp or bee velocity^2
velocity_stp	EQU	25			* wasp or bee velocity^2 step size

wasp_colour		EQU	$000000FF		* the wasp colour
bee_colour		EQU	$0000FF00		* the bee colour

pi20			EQU	$08			* rotation step size

x_size		EQU	$80			* environment box +/- x limits
y_size		EQU	$80			* environment box +/- y limits
z_size		EQU	$80			* environment box +/- z limits
border		EQU	$08			* border between environment limit and bee
							* or wasp travel

*************************************************************************************
*
* code start

	ORG	$1000

start
	MOVEQ		#10,d1			* OR mode drawing
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	MOVEQ		#17,d1			* enable double buffering
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	BSR		Initialise			* go setup everything

* main loop. everything is done in this loop, the overall speed of the loop is set by
* waiting for a fixed time to pass at the end of the loop

loop
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	MOVE.l	d1,-(sp)			* push time on the stack

	MOVEQ		#94,d0			* copy screen buffer to main
	TRAP		#15

	MOVE.w	#$FF00,d1			* clear the screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	BSR		screen_size			* test and handle widow size changes
	BSR		do_keys			* test and handle zoom and rotate keypresses
	BSR		do_keys_2			* test and handle bee/wasp parameter keypresses

	BSR		move_wasps			* move the wasps
	BSR		move_bees			* move the bees

	BSR		draw_status			* draw the status

	BSR		draw_3D			* go draw the bounding box

	BSR		draw_wasps			* draw the wasps
	BSR		draw_bees			* draw the bees

* now wait for a while. this limits the loop to ten redraws a second without which
* this would run way to fast if the machine is capable

	MOVE.l	(sp)+,d7			* restore time in 1/100 ths from stack
wait_100ms
	MOVEQ		#8,d0				* get time in 1/100 ths seconds
	TRAP		#15

	SUB.l		d7,d1				* subtract previous time from current time
	CMP.b		#$09,d1			* compare with 9/100ths
	BMI.s		wait_100ms			* loop if time not up yet

* if you want to see how fast it can go comment out the BMI.s above

	BRA		loop				* loop forever


*************************************************************************************
*
* check the [F2], [F3] and [F4] keys. set the screen size to 640 x 480, 800 x 600 or
* 1024 x 768 if the corresponding key has been pressed

screen_size
	MOVE.l	#$71007273,d1		* [F2], [], [F3] and [F4] keys
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	MOVE.l	d1,d2				* copy result
	BEQ.s		notscreen			* skip screen size if no F key

	MOVE.l	#$028001E0,d1		* set 640 x 480
	TST.l		d2				* test result
	BMI		setscreen			* if F2 go set window size

	MOVE.l	#$03200258,d1		* set 800 x 600
	TST.w		d2				* test result
	BMI		setscreen			* if F3 go set window size

							* else was F4 so ..
	MOVE.l	#$04000300,d1		* set 1024 x 768
setscreen
	CMP.l		screen_width(a3),d1	* compare with current screen size
	BEQ.s		notscreen			* if already set skip setting it now

	MOVE.l	d1,screen_width(a3)	* save the window size
	MOVEQ 	#33,d0			* get/set window size
	TRAP		#15

notscreen
	RTS


*************************************************************************************
*
* test and handle zoom and rotate keypresses

do_keys
	MOVE.l	#$00002122,d1		* [][][PgUp][PgDn] keys
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	TST.w		d1				* test [PgUp] result
	BPL.s		test_pgdn			* skip up if not pressed

	TST.b		d1				* test [PgDn] result
	BMI.s		keys_2			* if user hit [PgUp] and [PgDn] do nothing

	MOVE.w	eboy(a3),d2			* get environment origin y
	ASR.w		#2,d2				* / 4
	ADD.w		eboy(a3),d2			* add environment origin y
	CMP.w		#3000,d2			* compare with maximum
	BLE.s		save_eboy			* go save it if <= 3000

	MOVE.w	#3000,d2			* else set maximum
	BRA.s		save_eboy			* go save it

test_pgdn
	TST.b		d1				* test [PgDn] result
	BPL.s		keys_2			* skip down if not pressed

	MOVE.w	eboy(a3),d2			* get environment origin y
	ASR.w		#1,d2				* / 2
	ADD.w		eboy(a3),d2			* add environment origin y
	ASR.w		#1,d2				* / 2, d = 3/4 * eboy(a3)
	CMP.w		#200,d2			* compare with minimum
	BGE.s		save_eboy			* go save it if >= 200

	MOVE.w	#200,d2			* set maximum
save_eboy
	MOVE.w	d2,eboy(a3)			* save environment origin z

keys_2
	MOVE.l	#$00004257,d1		* [][][B][W] keys
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	TST.w		d1				* test the result
	BNE		exit_keys			* if [W] or [B] skip cursor key checks

	MOVE.l	#$26282527,d1		* [crup][crdn][crlt][crrt] keys
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	TST.w		d1				* test [crlt] result
	BPL.s		test_crrt			* skip left if not pressed

	TST.b		d1				* test [crrt] result
	BMI.s		test_crup			* if user hit [crlt] and [crrt] do nothing

	MOVEQ		#-pi20,d2			* set up step
	BRA.s		save_yaw			* go save it

test_crrt
	TST.b		d1				* test [crrt] result
	BPL.s		test_crup			* skip right if not pressed

	MOVEQ		#pi20,d2			* set up step
save_yaw
	ADD.w		d2,yaw(a3)			* add to environment rotation about y axis

test_crup
	SWAP		d1				* [crup][crdn] into low word
	TST.w		d1				* test [crup] result
	BPL.s		test_crdn			* if not [crup] go test [crdn]

	TST.b		d1				* test [crdn] result
	BMI.s		exit_keys			* if user hit [crup] and [crdn] do nothing

	MOVEQ		#-30,d2			* set up step
	ADD.w		eboz(a3),d2			* add environment origin z
	CMP.w		#-500,d2			* compare with minimum
	BGE.s		save_eboz			* go save it if >= -500

	MOVE.w	#-500,d2			* set minimum
	BRA.s		save_eboz			* go save it

test_crdn
	TST.b		d1				* test [crdn] result
	BPL.s		exit_keys			* if not [crdn] go do other keys

	MOVEQ		#30,d2			* set up step
	ADD.w		eboz(a3),d2			* add environment origin z
	CMP.w		#100,d2			* compare with maximum
	BLE.s		save_eboz			* go save it if <= 100

	MOVE.w	#100,d2			* set maximum
save_eboz
	MOVE.w	d2,eboz(a3)			* save environment origin z
exit_keys
	RTS


*************************************************************************************
*
* test and handle bee/wasp parameter keypresses. [S] toggles the status display on/off
*
* [B] plus [currt] to increase the bee maximum velocity
* [B] plus [curlt] to decrease the bee maximum velocity
* [B] plus [curup] to increase the bee maximum acceleration
* [B] plus [curdn] to decrease the bee maximum acceleration
*
* [W] plus [currt] to increase the wasp maximum velocity
* [W] plus [curlt] to decrease the wasp maximum velocity
* [W] plus [curup] to increase the wasp maximum acceleration
* [W] plus [curdn] to decrease the wasp maximum acceleration

do_keys_2
	MOVEQ		#$53,d1			* [][][][S] key
	MOVEQ		#0,d2				* clear longword
	MOVE.w	key_status(a3),d2		* get the last key status
	BSR		test_keys			* go test key
	BEQ		no_toggle_stat		* if no key press skip status toggle

	EOR.w		#-1,status(a3)		* toggle the status flag
no_toggle_stat
	MOVE.w	d2,key_status(a3)		* save the last key syatus

	MOVEQ		#'W',d1			* [][][][W] key
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	MOVE.l	d1,d2				* test the result and clear last key
	BEQ		not_wasp			* if not wasp go try bee

	MOVEQ		#0,d2				* clear longword
	MOVE.l	key_wupdnltrt(a3),d2	* get the last key status

	MOVE.w	wasp_velocity_max(a3),d5
							* get wasp acceleration
	MOVE.w	wasp_acceleration(a3),d6
							* get wasp velocity

	BSR		test_updnltrt		* test the [crup][crdn][crlt][crrt] keys
	MOVE.w	d5,wasp_velocity_max(a3)
							* save wasp acceleration
	MOVE.w	d6,wasp_acceleration(a3)
							* save wasp velocity
not_wasp
	MOVE.l	d2,key_wupdnltrt(a3)	* save the last key status

	MOVEQ		#'B',d1			* [][][][B] key
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	TST.b		d1				* test the result
	BEQ		not_bee			* if not bee just exit

	MOVEQ		#0,d2				* clear longword
	MOVE.l	key_bupdnltrt(a3),d2	* get the last key status

	MOVE.w	bee_velocity_max(a3),d5	* get bee acceleration
	MOVE.w	bee_acceleration(a3),d6	* get bee velocity

	BSR		test_updnltrt		* test the [crup][crdn][crlt][crrt] keys

	MOVE.w	d5,bee_velocity_max(a3)	* save bee acceleration
	MOVE.w	d6,bee_acceleration(a3)	* save bee velocity
not_bee
	MOVE.l	d2,key_bupdnltrt(a3)	* save the last key syatus
	RTS


*************************************************************************************
*
* test the [crup][crdn][crlt][crrt] keys. increase or decrease the acceleration or
* velocity, check they are in range and return the in range values. enter with
* velocity in d5.w and acceleration in d6.w, returns modified velocity in d5.w and
* acceleration in d6.w

test_updnltrt
	MOVE.l	#$26282527,d1		* [crup][crdn][crlt][crrt] keys
	BSR		test_keys			* go test key

	MOVEQ		#0,d3				* clear delta velocity
	MOVEQ		#0,d4				* clear delta acceleration

	TST.w		d1				* test [crlt] key pressed
	BPL		no_crlt			* if not [crlt] skip delta set

	MOVE.w	#-velocity_stp,d3		* set -ve delta bee velocity
no_crlt
	TST.b		d1				* test [crrt] key pressed
	BPL		no_crrt			* if not [crrt] skip delta set

	MOVE.w	#velocity_stp,d3		* set +ve delta bee velocity
no_crrt
	SWAP		d1				* [crup][crdn] to low word
	TST.w		d1				* test [crup] key pressed
	BPL		no_crup			* if not [crup] skip delta set

	MOVE.w	#acceleration_stp,d4	* set +ve delta acceleration
no_crup
	TST.b		d1				* test [crdn] key pressed
	BPL		no_crdn			* if not [crdn] skip delta set

	MOVE.w	#-acceleration_stp,d4	* set -ve delta acceleration
no_crdn
	ADD.w		d5,d3				* make new acceleration
	ADD.w		d6,d4				* make new velocity

	CMP.w		#velocity_max,d3		* compare new velocity with max
	BGT		velocity_bad		* if > max skip velocity set

	CMP.w		#velocity_min,d3		* compare new velocity with min
	BLT		velocity_bad		* if < min skip velocity set

	MOVE.w	d3,d5				* copy new velocity
velocity_bad
	CMP.w		#acceleration_max,d4	* compare new acceleration with max
	BGT		acceleration_bad		* if > max skip acceleration set

	CMP.w		#acceleration_min,d4	* compare new acceleration with min
	BLT		acceleration_bad		* if < min skip acceleration set

	MOVE.w	d4,d6				* copy new acceleration
acceleration_bad
	RTS


*************************************************************************************
*
* test key(s) d1.l, last state in d2.l. returns new pressed in d1.l and new state in
* d2.l

test_keys
	MOVE.l	d0,-(sp)			* save register
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	MOVE.l	d1,d0				* copy the result, each byte is $00 if key up,
							* $FF if key down
	EOR.l		d2,d1				* compare with last key(s) state, each byte
							* is now $FF if a key has changed or $FF if
							* a key has not changed
	MOVE.l	d0,d2				* save this result as last
	MOVE.l	(sp)+,d0			* restore register
	AND.l		d2,d1				* make each byte $FF if key just pressed or
							* $00 if key not just pressed, set Zb status
	RTS


*************************************************************************************
*
* draw the current settings

draw_status
	TST.w		status(a3)			* test the status flag
	BEQ		exit_status			* if not on just exit

	MOVE.w	#$0201,d1			* set 2,1
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	LEA		string(pc),a1		* point to string
	MOVEQ		#14,d0			* display the NULL terminated string at (a1)
	TRAP		#15

	MOVE.w	#$1601,d1			* set 22,1
	MOVE.w	bee_velocity_max(a3),d2	* maximum bee (x^2 + y^2) velocity
	BSR		display_number		* display d1.l in base 10

	MOVE.w	#$1602,d1			* set 22,2
	MOVE.w	bee_acceleration(a3),d2	* get the bee acceleration
	BSR		display_number		* display d1.l in base 10

	MOVE.w	#$1603,d1			* set 22,3
	MOVE.w	wasp_velocity_max(a3),d2
							* maximum wasp (x^2 + y^2) velocity
	BSR		display_number		* display d1.l in base 10

	MOVE.w	#$1604,d1			* set 22,4
	MOVE.w	wasp_acceleration(a3),d2
							* get the wasp acceleration
	BSR		display_number		* display d1.l in base 10

exit_status
	RTS

string
	dc.b	'Bee velocity      :       [B] [curlt]- [currt]+',$0D,$0A
	dc.b	'  Bee acceleration  :       [B] [curdn]- [curup]+',$0D,$0A
	dc.b	'  Wasp velocity     :       [W] [curlt]- [currt]+',$0D,$0A
	dc.b	'  Wasp acceleration :       [W] [curdn]- [curup]+',0
	ds.w	0


*************************************************************************************
*
* output number, cursor position in d1.w, number in d2.w

display_number
	CMP.w		#1000,d2			* compare with 1000
	BCC		print_number		* if >= 100 go output the number

	ADD.w		#$0100,d1			* else increment the x position

	CMP.w		#100,d2			* compare with 100
	BCC		print_number		* if >= 100 go output the number

	ADD.w		#$0100,d1			* else increment the x position

	CMP.w		#10,d2			* compare with 10
	BCC		print_number		* if >= 10 go output the number

	ADD.w		#$0100,d1			* else increment the x position
print_number
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	MOVEQ		#0,d1				* clear longword
	MOVE.w	d2,d1				* copy the number to display
	MOVEQ		#$0A,d2			* set base 10
	MOVEQ		#15,d0			* display d1.l in base d2.b
	TRAP		#15

	RTS


*************************************************************************************
*
* draw the 3D environment bounding box at +/- x,y,z

draw_3D
	BSR		sincos			* get scaled sin(yaw) in d2, cos(yaw) in d3

	MOVE.w	#-x_size,d4			* set point x
	MOVE.w	#-y_size,d5			* set point y
	MOVE.w	#-z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x1(a3)			* save screen x co-ordinate
	MOVE.w	d1,y1(a3)			* save screen y co-ordinate

	MOVE.w	#x_size,d4			* set point x
	MOVE.w	#-y_size,d5			* set point y
	MOVE.w	#-z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x2(a3)			* save screen x co-ordinate
	MOVE.w	d1,y2(a3)			* save screen y co-ordinate

	MOVE.w	#x_size,d4			* set point x
	MOVE.w	#y_size,d5			* set point y
	MOVE.w	#-z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x3(a3)			* save screen x co-ordinate
	MOVE.w	d1,y3(a3)			* save screen y co-ordinate

	MOVE.w	#-x_size,d4			* set point x
	MOVE.w	#y_size,d5			* set point y
	MOVE.w	#-z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x4(a3)			* save screen x co-ordinate
	MOVE.w	d1,y4(a3)			* save screen y co-ordinate

	MOVE.w	#-x_size,d4			* set point x
	MOVE.w	#-y_size,d5			* set point y
	MOVE.w	#z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x5(a3)			* save screen x co-ordinate
	MOVE.w	d1,y5(a3)			* save screen y co-ordinate

	MOVE.w	#x_size,d4			* set point x
	MOVE.w	#-y_size,d5			* set point y
	MOVE.w	#z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x6(a3)			* save screen x co-ordinate
	MOVE.w	d1,y6(a3)			* save screen y co-ordinate

	MOVE.w	#x_size,d4			* set point x
	MOVE.w	#y_size,d5			* set point y
	MOVE.w	#z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x7(a3)			* save screen x co-ordinate
	MOVE.w	d1,y7(a3)			* save screen y co-ordinate

	MOVE.w	#-x_size,d4			* set point x
	MOVE.w	#y_size,d5			* set point y
	MOVE.w	#z_size,d6			* set point z
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d0,x8(a3)			* save screen x co-ordinate
	MOVE.w	d1,y8(a3)			* save screen y co-ordinate

	MOVE.l	#$0000FF,d1			* set red/green/blue
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVE.w	x1(a3),d1			* get line x1
	MOVE.w	y1(a3),d2			* get line y1
	MOVE.w	x2(a3),d3			* get line x2
	MOVE.w	y2(a3),d4			* get line y2
	MOVEQ		#84,d0			* set draw x1,y1 to x2,y2
	TRAP		#15

	MOVE.l	#$00FF00,d1			* set red/green/blue
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVE.w	x4(a3),d1			* get line x1
	MOVE.w	y4(a3),d2			* get line y1
	MOVE.w	x1(a3),d3			* get line x2
	MOVE.w	y1(a3),d4			* get line y2
	MOVEQ		#84,d0			* set draw x1,y1 to x2,y2
	TRAP		#15

	MOVE.l	#$FF0000,d1			* set red/green/blue
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVE.w	x1(a3),d1			* get line x1
	MOVE.w	y1(a3),d2			* get line y1
	MOVE.w	x5(a3),d3			* get line x2
	MOVE.w	y5(a3),d4			* get line y2
	MOVEQ		#84,d0			* set draw x1,y1 to x2,y2
	TRAP		#15

	MOVE.l	#$808080,d1			* set red/green/blue
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVE.w	x2(a3),d1			* get line x1
	MOVE.w	y2(a3),d2			* get line y1
	MOVE.w	x3(a3),d3			* get line x2
	MOVE.w	y3(a3),d4			* get line y2
	MOVEQ		#84,d0			* set draw x1,y1 to x2,y2
	TRAP		#15

	MOVE.w	x3(a3),d1			* get line x1
	MOVE.w	y3(a3),d2			* get line y1
	MOVE.w	x4(a3),d3			* get line x2
	MOVE.w	y4(a3),d4			* get line y2
	TRAP		#15

	MOVE.w	x5(a3),d1			* get line x1
	MOVE.w	y5(a3),d2			* get line y1
	MOVE.w	x6(a3),d3			* get line x2
	MOVE.w	y6(a3),d4			* get line y2
	TRAP		#15

	MOVE.w	x6(a3),d1			* get line x1
	MOVE.w	y6(a3),d2			* get line y1
	MOVE.w	x7(a3),d3			* get line x2
	MOVE.w	y7(a3),d4			* get line y2
	TRAP		#15

	MOVE.w	x7(a3),d1			* get line x1
	MOVE.w	y7(a3),d2			* get line y1
	MOVE.w	x8(a3),d3			* get line x2
	MOVE.w	y8(a3),d4			* get line y2
	TRAP		#15

	MOVE.w	x8(a3),d1			* get line x1
	MOVE.w	y8(a3),d2			* get line y1
	MOVE.w	x5(a3),d3			* get line x2
	MOVE.w	y5(a3),d4			* get line y2
	TRAP		#15

	MOVE.w	x4(a3),d1			* get line x1
	MOVE.w	y4(a3),d2			* get line y1
	MOVE.w	x8(a3),d3			* get line x2
	MOVE.w	y8(a3),d4			* get line y2
	TRAP		#15

	MOVE.w	x3(a3),d1			* get line x1
	MOVE.w	y3(a3),d2			* get line y1
	MOVE.w	x7(a3),d3			* get line x2
	MOVE.w	y7(a3),d4			* get line y2
	TRAP		#15

	MOVE.w	x2(a3),d1			* get line x1
	MOVE.w	y2(a3),d2			* get line y1
	MOVE.w	x6(a3),d3			* get line x2
	MOVE.w	y6(a3),d4			* get line y2
	TRAP		#15

	RTS


*************************************************************************************
*
* convert co-ordinate sets. first the object co-ordinates x,y,z, held in d4,d5,d6,
* are converted to world co-ordinates by multiplying x,y,z by the object yaw and then
* adding the object's world co-ordinate origin.
*
* returns xa,ya in d0,d1 and wx,wy,wz in d4,d5,d6

* wx& = ebox& + (x& * cosa) - (y& * sina)
* wy& = eboy& + (x& * sina) + (y& * cosa)
* wz& = eboz& + z&

make_xy
	MOVE.l	d7,-(sp)			* save register
	MOVE.w	d4,d0				* copy x co-ordinate
	MOVE.w	d5,d1				* copy y co-ordinate

							* wx = ebox + x * cos(yaw) - y * sin(yaw)
	MULS.w	d3,d4				* cos * x co-ordinate
	MULS.w	d2,d1				* sin * y co-ordinate
	SUB.l		d1,d4				* = x * cos - y * sin
	ADD.l		d4,d4				* scale 0 to 1
	SWAP		d4				* only need x high word
	ADD.w		ebox(a3),d4			* + ebox

							* wy = eboy + x * sin(yaw) + y * cos(yaw)
	MULS.w	d2,d0				* sin * x co-ordinate
	MULS.w	d3,d5				* cos * y co-ordinate
	ADD.l		d0,d5				* = x * sin + y * cos
	ADD.l		d5,d5				* scale 0 to 1
	SWAP		d5				* only need y high word
	ADD.w		eboy(a3),d5			* + eboy

							* wz = eboz + z
	ADD.w		eboz(a3),d6			* + eboz

* now the world object co-ordinates are transformed to the screen co-ordinates

	MOVE.w	d4,d0				* copy x co-ordinate
	MOVE.w	d5,d1				* copy y co-ordinate

* xa& = x centre - x size * wx& / wy&
* ya& = y centre - x size * wz& / wy&

	MULS.w	screen_width(a3),d0	* multiply and make longword
	NEG.l		d0				* change sign for - field of view
	DIVS.w	d1,d0				* divide by distance
	MOVE.w	screen_width(a3),d7	* get the screen width
	ASR.w		#1,d7				* / 2 to get x centre
	ADD.w		d7,d0				* d0 is now screen x

	MOVE.w	d6,d7				* copy z co-ordinate

	MULS.w	screen_width(a3),d7	* multiply and make longword
	NEG.l		d7				* change sign for - field of view
	DIVS.w	d1,d7				* divide by distance
	MOVE.w	screen_height(a3),d1	* get the screen height
	ASR.w		#1,d1				* / 2 to get y centre
	ADD.w		d7,d1				* d1 is now screen y

	MOVE.l	(sp)+,d7			* restore register
	RTS


*************************************************************************************
*
* setup stuff.

Initialise
	LEA		variables(pc),a3		* variables base address

	LEA		wasp_objects(pc),a6	* wasp objects base address
	LEA		bee_objects(pc),a5	* bee objects base address

	MOVE.l	#$31415927,PRNlword(a3)	* set PRNG seed, this can be any value but 0

	MOVE.w	#pi20*$1B,yaw(a3)		* set initial environment rotation about y axis

	MOVE.w	#0,ebox(a3)			* set initial environment origin x
	MOVE.w	#650,eboy(a3)		* set initial environment origin y
	MOVE.w	#10,eboz(a3)		* set initial environment origin z

	MOVEQ		#0,d0				* clear longword
	MOVE.w	d0,status(a3)		* clear the show status flag
	MOVE.w	d0,key_status(a3)		* clear the status key last state
	MOVE.l	d0,key_wupdnltrt(a3)	* clear the wasp key last state
	MOVE.l	d0,key_bupdnltrt(a3)	* clear the bee key last state

	MOVE.w	#4,wasp_acceleration(a3)
							* how fast the wasps can accelerate
	MOVE.w	#4,bee_acceleration(a3)	* how fast the bees can accelerate

	MOVE.w	#50,wasp_velocity_max(a3)
							* maximum wasp (x^2 + y^2 + z^2) velocity
	MOVE.w	#150,bee_velocity_max(a3)
							* maximum bee (x^2 + y^2 + z^2) velocity

	MOVE.l	#$028001E0,d1		* set default window size to 640 x 480
	MOVE.l	d1,screen_width(a3)	* save the window size
	MOVEQ 	#33,d0			* get/set window size
	TRAP		#15

*************************************************************************************
*
* generate the wasps

make_wasps
	MOVE.w	#(number_of_wasps-1)*wasp_size,d7
							* set loop count/index
make_wasps_loop
	MOVE.w	#x_size-border,d2		* get the x maximum
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,x_pos(a6,d7.w)		* set the wasp x co-ordinate

	MOVE.w	#y_size-border,d2		* get the y maximum
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,y_pos(a6,d7.w)		* set the wasp y co-ordinate

	MOVE.w	#z_size-border,d2		* get the z maximum
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,z_pos(a6,d7.w)		* set the wasp z co-ordinate

	MOVE.w	wasp_acceleration(a3),d2
							* get the wasp acceleration
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,x_vel(a6,d7.w)		* set the wasp x velocity

	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,y_vel(a6,d7.w)		* set the wasp y velocity

	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,z_vel(a6,d7.w)		* set the wasp z velocity

	SUB.w		#wasp_size,d7		* subtract one wasp structure size
	BPL		make_wasps_loop		* loop if more wasps to do



*************************************************************************************
*
* generate the bees

make_bees
	MOVE.w	#(number_of_bees-1)*bee_size,d7
							* set loop count/index
make_bees_loop
	MOVE.w	#x_size-border,d2		* get the x maximum
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,x_pos(a5,d7.w)		* set the bee x co-ordinate

	MOVE.w	#y_size-border,d2		* get the y maximum
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,y_pos(a5,d7.w)		* set the bee y co-ordinate

	MOVE.w	#z_size-border,d2		* get the z maximum
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,z_pos(a5,d7.w)		* set the bee z co-ordinate

	MOVE.w	bee_acceleration(a3),d2	* get the bee acceleration
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,x_vel(a5,d7.w)		* set the bee x velocity

	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,y_vel(a5,d7.w)		* set the bee y velocity

	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,z_vel(a5,d7.w)		* set the bee z velocity

	SUB.w		#bee_size,d7		* subtract one bee structure size
	BPL		make_bees_loop		* loop if more bees to do

	RTS


*************************************************************************************
*
* move the wasps. the wasps wander around randomly constrained by three parameters.
* wasp_acceleration is the maximum ammount by which the x, y or z velocity can change
* in one go (total change can be up to SQR(3*wasp_acceleration^2), wasp_velocity_max
* is the maximum linear velocity, (x^2 + y^2 + z^2 velocity), that the wasp can reach
* and lastly the environment bounding box limits minus a border which the wasps will
* bounce off

move_wasps
	MOVE.w	#(number_of_wasps-1)*wasp_size,d7
							* set loop count/index
move_wasps_loop
	MOVE.w	x_pos(a6,d7.w),x_old(a6,d7.w)
							* copy current x to previous x co-ordinate
	MOVE.w	y_pos(a6,d7.w),y_old(a6,d7.w)
							* copy current y to previous y co-ordinate
	MOVE.w	z_pos(a6,d7.w),z_old(a6,d7.w)
							* copy current z to previous z co-ordinate

							* generate the new delta velocities
	MOVE.w	wasp_acceleration(a3),d2
							* get the wasp acceleration
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,d4				* copy the delta x velocity
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,d5				* copy the delta y velocity
	BSR		rand_pm_d2			* generate a random number in the range -(d2.w
							* - 1) to (d2.w - 1), returned in d0.w
	MOVE.w	d0,d6				* copy the delta z velocity

	MOVEQ		#0,d3				* clear longword
	MOVE.w	wasp_velocity_max(a3),d3
							* get the wasp (x^2 + y^2 + z^2) velocity max
	LEA		(a6,d7.w),a1		* get the wasp pointer
	BSR		velocity_change		* apply the delta x,y,z velocity to the insect

	SUB.w		#wasp_size,d7		* subtract one wasp structure size
	BPL		move_wasps_loop		* loop if more wasps to do

	RTS


*************************************************************************************
*
* move the bees. the bees move towards the nearest wasp. this is done by trying to
* reduce SQR(delta ^2 + delta y^2 + delta z^2) to a minimum which, as the bees are
* in constant motion, means that they effectively orbit the closest wasp.
*
* like the wasps they are constrained by three parameters. bee_acceleration is the
* maximum ammount by which the x, y or z velocity can change in one go (total change
* can be up to SQR(3*bee_acceleration^2), bee_velocity_max is the maximum linear
* velocity, (x^2 + y^2 + z^2 velocity), that the bee can reach and lastly the
* environment bounding box limits minus a border which the bees will bounce off

move_bees
	MOVE.w	#(number_of_bees-1)*bee_size,d7
							* set loop count/index
move_bees_loop
	MOVE.w	x_pos(a5,d7.w),x_old(a5,d7.w)
							* copy current x to previous x co-ordinate
	MOVE.w	y_pos(a5,d7.w),y_old(a5,d7.w)
							* copy current y to previous y co-ordinate
	MOVE.w	z_pos(a5,d7.w),z_old(a5,d7.w)
							* copy current z to previous z co-ordinate

* find the distance^2 and delta x, y and z to the nearest wasp

	MOVE.w	d7,-(sp)			* run out of registers so save the outer
							* loop index

	MOVE.w	z_pos(a5,d7.w),-(sp)	* stack this bee's z position
	MOVE.w	y_pos(a5,d7.w),-(sp)	* stack this bee's y position
	MOVE.w	x_pos(a5,d7.w),-(sp)	* stack this bee's x position

	MOVEQ		#-1,d0			* set the smallest distance^2
	MOVE.w	#(number_of_wasps-1)*wasp_size,d7
							* set loop count/index
check_wasps_loop
	MOVEM.w	(sp),d1-d3			* get this bee's x,y,z position

	SUB.w		x_pos(a6,d7.w),d1		* get this wasp's delta x position
	SUB.w		y_pos(a6,d7.w),d2		* get this wasp's delta y position
	SUB.w		z_pos(a6,d7.w),d3		* get this wasp's delta z position
	MOVEM.w	d1-d3,-(sp)			* save delta x,y,z

	MULS.w	d1,d1				* new delta x^2
	MULS.w	d2,d2				* new delta y^2
	MULS.w	d3,d3				* new delta z^2
	ADD.l		d1,d2				* new distance^2 = (delta x^2) + (delta y^2)
	ADD.l		d2,d3				* new distance^2 = (delta x^2) + (delta y^2)
							*		     + (delta z^2)
	CMP.l		d3,d0				* compare new distance^2 with last distance^2
	BCS		no_new_dxy2			* if new distance^2 bigger skip update

							* this wasp is closer so use the distance^2
							* and delta x, delta y to this wasp
	MOVE.l	d3,d0				* make distance^2 = new distance^2
	MOVEM.w	(sp),d4-d6			* make delta x,y,z = new delta x,y,z
no_new_dxy2
	ADDQ.l	#6,sp				* dump delta x,y,z from stack

	SUB.w		#wasp_size,d7		* subtract one wasp structure size
	BPL		check_wasps_loop		* loop if more wasps to check

	ADDQ.l	#6,sp				* dump this bee's x,y,z position from stack

	NEG.w		d4				* change the delta x,y and z ..
	NEG.w		d5				* .. signs so that the bee ..
	NEG.w		d6				* .. moves towards the wasp

	BSR		root_d0			* calculate SQR(distance^2)
	ADDQ.w	#1,d0				* ensure distance <> 0. this serves to remove
							* the chance of a divide by zero error and to
							* ensure the bee continually orbits the wasp

	MOVE.w	(sp)+,d7			* restore outer loop index

* now the distance, delta x,y and z to the nearest wasp are known change the x,y,z
* velocity towards it and move the bee

	MOVE.w	bee_acceleration(a3),d1	* set bee acceleration
	MULS.w	d1,d4				* bee acceleration * delta x
	DIVS.w	d0,d4				* bee acceleration * delta x / distance
	MULS.w	d1,d5				* bee acceleration * delta y
	DIVS.w	d0,d5				* bee acceleration * delta y / distance
	MULS.w	d1,d6				* bee acceleration * delta z
	DIVS.w	d0,d6				* bee acceleration * delta z / distance

	MOVEQ		#0,d3				* clear longword
	MOVE.w	bee_velocity_max(a3),d3	* set the bee (x^2 + y^2 + z^2) velocity max

	LEA		(a5,d7.w),a1		* get the bee pointer
	BSR		velocity_change		* apply the delta x,y,z velocity to the insect

	SUB.w		#bee_size,d7		* subtract one bee structure size
	BPL		move_bees_loop		* loop if more bees to do

	RTS


*************************************************************************************
*
* draw the wasps

draw_wasps
	MOVE.l	#wasp_colour,d1		* set the pen to the wasp colour
	MOVE.w	#(number_of_wasps-1)*wasp_size,d7
							* set loop count/index
	LEA		(a6),a1			* set the insect pointer to the wasps
	MOVE.w	#wasp_size,-(sp)		* push wasp structure size
	BRA.s		draw_insects		* go draw the wasps and return


*************************************************************************************
*
* draw the bees

draw_bees
	MOVE.l	#bee_colour,d1		* set the pen to the bee colour
	MOVE.w	#(number_of_bees-1)*bee_size,d7
							* set loop count/index
	LEA		(a5),a1			* set the insect pointer to the bees
	MOVE.w	#bee_size,-(sp)		* push bee structure size


*************************************************************************************
*
* draw either wasps or bees

draw_insects
	MOVEQ		#80,d0			* set the pen colour
	TRAP		#15
draw_insect_loop
	BSR		sincos			* get scaled sin(yaw) in d2, cos(yaw) in d3

	MOVE.w	x_old(a1,d7.w),d4		* get x old co-ordinate
	MOVE.w	y_old(a1,d7.w),d5		* get y old co-ordinate
	MOVE.w	z_old(a1,d7.w),d6		* get z old co-ordinate
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVEM.w	d0-d1,-(sp)			* save screen x,y co-ordinates

	MOVE.w	x_pos(a1,d7.w),d4		* get x position co-ordinate
	MOVE.w	y_pos(a1,d7.w),d5		* get y position co-ordinate
	MOVE.w	z_pos(a1,d7.w),d6		* get z position co-ordinate
	BSR		make_xy			* convert to world x,y,z and screen x,y
	MOVE.w	d1,d2				* copy y co-ordinate
	MOVE.w	d0,d1				* copy x co-ordinate

	MOVEM.w	(sp)+,d3-d4			* restore screen x,y co-ordinates
	MOVEQ		#84,d0			* draw a line from x1,y1 to x2,y2
	TRAP		#15

	SUB.w		(sp),d7			* subtract one insect structure size
	BPL		draw_insect_loop		* loop if more insects to do

	ADDQ.w	#2,sp				* dump insect size from stack
	RTS


*************************************************************************************
*
* apply the delta x,y,z velocity to the insect. the deltas are in d4.w, d5.w and d6.w,
* a1 points to the bee or wasp to be changed. the maximum velocity is in d3.l.

* calculate the new velocity^2 and compare with the maximum velocity^2. If velocity^2 >
* maximum velocity^2 then scale the x,y and z velocities by SQR(maximun velocity^2 /
* velocity^2)

velocity_change
	MOVE.w	x_vel(a1),d0		* get the insect x velocity
	ADD.w		d4,d0				* add the delta x velocity
	MOVE.w	d0,d4				* copy the new insect x velocity
	MULS.w	d0,d0				* make new x^2

	MOVE.w	y_vel(a1),d1		* get the insect y velocity
	ADD.w		d5,d1				* add the delta y velocity
	MOVE.w	d1,d5				* copy the new insect y velocity
	MULS.w	d1,d1				* make new y^2

	MOVE.w	z_vel(a1),d2		* get the insect z velocity
	ADD.w		d6,d2				* add the delta z velocity
	MOVE.w	d2,d6				* copy the new insect z velocity
	MULS.w	d2,d2				* make new z^2

	ADD.l		d1,d0				* x^2 + y^2
	ADD.l		d2,d0				* x^2 + y^2 + z^2
	CMP.l		d0,d3				* compare the new velocity^2 with the maximum
	BCC		velocity_ok			* if new <= maximum then skip the scaling

* do x, y, z * scale where scale = SQR(max velocity^2 / new velocity^2)

	SWAP		d3				* maximum velocity^2 * $10000
	DIVU.w	d0,d3				* divide by the new velocity^2
	MOVEQ		#0,d0				* clear longword
	MOVE.w	d3,d0				* copy max velocity^2 / new velocity^2
	BSR		root_d0			* calculate SQR(max velocity^2 / new velocity^2)
	LSR.l		#1,d0				* / 2 as we're going to do signed multiplication
	SWAP		d0				* get the fractional result

	MOVE.w	d4,d3				* copy the x velocity
	BSR.s		scale_velocity		* scale it
	MOVE.w	d3,d4				* restore the x velocity
	MOVE.w	d5,d3				* copy the y velocity
	BSR.s		scale_velocity		* scale it
	MOVE.w	d3,d5				* restore the y velocity
	MOVE.w	d6,d3				* copy the z velocity
	BSR.s		scale_velocity		* scale it
	MOVE.w	d3,d6				* restore the z velocity

velocity_ok
	MOVE.w	d4,d0				* copy the new x velocity
	MOVE.w	x_pos(a1),d1		* get the insect x position
	MOVE.w	#x_size-border,d4		* get the x position max
	BSR		check_position		* calculate and check next x position
	MOVE.w	d0,x_vel(a1)		* save the insect x velocity
	MOVE.w	d1,x_pos(a1)		* save the insect x position

	MOVE.w	d5,d0				* copy the new y velocity
	MOVE.w	y_pos(a1),d1		* get the insect y position
	MOVE.w	#y_size-border,d4		* get the y position max
	BSR		check_position		* calculate and check next y position
	MOVE.w	d0,y_vel(a1)		* save the insect y velocity
	MOVE.w	d1,y_pos(a1)		* save the insect y position

	MOVE.w	d6,d0				* copy the new z velocity
	MOVE.w	z_pos(a1),d1		* get the insect z position
	MOVE.w	#z_size-border,d4		* get the z position max
	BSR		check_position		* calculate and check next z position
	MOVE.w	d0,z_vel(a1)		* save the insect z velocity
	MOVE.w	d1,z_pos(a1)		* save the insect z position

	RTS


*************************************************************************************
*
* scale the velocity. enter with the velocity to scale in d3. all this jumping through
* hoops to do a 16 bit unsigned * 16 bit signed multiply is needed as without it the
* cumulative errors from dropping the least significant bit become visible

scale_velocity
	MOVEQ		#0,d1				* clear longword
	MOVE.w	d3,d1				* copy the velocity
	MULS.w	d0,d3				* x velocity * scale
	ADD.l		d3,d3				* correct for earlier / 2
	TST.l		d0				* test original lowest bit
	BPL		scale_done			* skip fractional add if zero

	ADD.l		d1,d3				* add velocity
scale_done
	SWAP		d3				* high word is integer part
	RTS


*************************************************************************************
*
* check position and return limited version. put the position parameter to check in
* d1.w, the velocity in d0.w and put the upper limit in d4.w. the limited position is
* returned in d1.w and the possibly inverted velocity component is returned in d0.w

check_position
	MOVE.l	d5,-(sp)			* save register
	MOVE.w	d0,d5				* copy the velocity
	ADD.w		d1,d5				* add the position
	BPL		check_high			* if >= 0 skip the position negate

	NEG.w		d5				* negate position to do positive compare
check_high
	CMP.w		d5,d4				* compare new position with the limit
	BCC		check_ok			* if < the limit skip the velocity negate

	NEG.w		d0				* else negate the velocity
check_ok
	ADD.w		d0,d1				* add the velocity to the position
	MOVE.l	(sp)+,d5			* restore register
	RTS


*************************************************************************************
*
* perform SQR(d0.l) return the result in d0.w

root_d0
	MOVEM.l	d2-d5,-(sp)			* save registers
	MOVEQ		#0,d5				* clear root
	MOVEQ		#0,d2				* clear remainder

	MOVEQ		#$1F,d3			* $1F for DBF, 64 pairs of bits to do for
							* a 32 bit result
root_loop
	ADD.l		d0,d0				* shift highest bit of number ..
	ADDX.l	d2,d2				* .. into remainder .. never overflows
	ADD.l		d5,d5				* root = root * 2 .. never overflows
	ADD.l		d0,d0				* shift highest bit of number ..
	ADDX.l	d2,d2				* .. into remainder .. never overflows

	MOVE.l	d5,d4				* copy root
	ADD.l		d4,d4				* 2n
	ADDQ.l	#1,d4				* 2n+1

	CMP.l		d4,d2				* compare 2n+1 to remainder
	BCS.s		root_ns			* skip sub if remainder smaller

	SUB.l		d4,d2				* subtract temp from remainder
	ADDQ.l	#1,d5				* increment root
root_ns
	DBF		d3,root_loop		* loop if not all done

	SWAP		d5				* shift for correct integer result
	MOVE.l	d5,d0				* copy result
	MOVEM.l	(sp)+,d2-d5			* restore registers
	RTS


*************************************************************************************
*
* generate a random value in d0.w in range -(d2.w - 1) to +(d2.w - 1)

rand_pm_d2
	BSR		random_n			* generate a random number
	MOVE.w	PRNlword(a3),d0		* get a random word
	EXT.l		d0				* make into longword. doing this instead of
							* just getting a longword ensures that the
							* following DIVS never overflows
	DIVS.w	d2,d0				* divide by range
	SWAP		d0				* use the signed remainder
	RTS


*************************************************************************************
*
* This is the code that generates the pseudo random sequence. A seed word located in
* PRNlword(a3) is loaded into a register before being operated on to generate the
* next number in the sequence. This number is then saved as the seed for the next
* time it's called.
*
* This code is adapted from the 32 bit version of RND(n) used in EhBASIC68. Taking
* the 19th next number is slower but helps to hide the shift and add nature of this
* generator as can be seen from analysing the output.

random_n
	MOVEM.l	d0-d2,-(sp)			* save d0, d1 and d2
	MOVE.l	PRNlword(a3),d0		* get current seed longword
	MOVEQ		#$AF-$100,d1		* set EOR value
	MOVEQ		#18,d2			* do this 19 times
Ninc0
	ADD.l		d0,d0				* shift left 1 bit
	BCC.s		Ninc1				* if bit not set skip feedback

	EOR.b		d1,d0				* do Galois LFSR feedback
Ninc1
	DBF		d2,Ninc0			* loop

	MOVE.l	d0,PRNlword(a3)		* save back to seed longword
	MOVEM.l	(sp)+,d0-d2			* restore d0, d1 and d2

	RTS


*************************************************************************************
*
* get sin(yaw) in d2, cos(yaw) in d3.

sincos
	MOVE.w	yaw(a3),d0			* get environment orientation
	BSR.s		sin_d0			* get SIN(d0) in d0
	MOVE.w	d0,d2				* copy it

	MOVE.w	yaw(a3),d0			* get environment orientation
	BSR.s		cos_d0			* get COS(d0) in d0
	MOVE.w	d0,d3				* copy it
	RTS

*************************************************************************************
*
* get COS(d0) in d0. d0 is a nine bit value representing a full circle with the value
* increasing as you turn widdershins

cos_d0
	ADD.w		#$80,d0			* add 1/4 rotation

*************************************************************************************
*
* get SIN(d0) in d0. d0 is a nine bit value representing a full circle with the value
* increasing as you turn widdershins

sin_d0
	BTST		#8,d0				* test angle sign
	BEQ.s		cossin_d0			* just get SIN/COS and return if +ve

	BSR.s		cossin_d0			* else get SIN/COS
	NEG.w		d0				* now do twos complement
	RTS

* get d0 from SIN/COS table

cossin_d0
	TST.b		d0				* test for >= $80
	BPL.s		a_was_less			* branch if < 1/4 circle

	NEG.b		d0				* wrap $81 to $FF to $7F to $01
a_was_less
	AND.w		#$FF,d0			* ensure word high byte clear
	ADD.w		d0,d0				* * 2 bytes per word value
	LEA		sin_cos(pc),a0		* get table base address
	MOVE.w	(a0,d0.w),d0		* get SIN/COS value
	RTS

*************************************************************************************
*
* SIN/COS table, returns values between $0000 and $7FFF. the last value should be
* $8000 but that can cause an overflow in the word length calculations and it's
* easier to fudge the table a bit. no one will ever notice.

sin_cos
	dc.w	$0000,$0192,$0324,$04B6,$0648,$07D9,$096B,$0AFB
	dc.w	$0C8C,$0E1C,$0FAB,$113A,$12C8,$1455,$15E2,$176E
	dc.w	$18F9,$1A83,$1C0C,$1D93,$1F1A,$209F,$2224,$23A7
	dc.w	$2528,$26A8,$2827,$29A4,$2B1F,$2C99,$2E11,$2F87
	dc.w	$30FC,$326E,$33DF,$354E,$36BA,$3825,$398D,$3AF3
	dc.w	$3C57,$3DB8,$3F17,$4074,$41CE,$4326,$447B,$45CD
	dc.w	$471D,$486A,$49B4,$4AFB,$4C40,$4D81,$4EC0,$4FFB
	dc.w	$5134,$5269,$539B,$54CA,$55F6,$571E,$5843,$5964
	dc.w	$5A82,$5B9D,$5CB4,$5DC8,$5ED7,$5FE4,$60EC,$61F1
	dc.w	$62F2,$63EF,$64E9,$65DE,$66CF,$67BD,$68A7,$698C
	dc.w	$6A6E,$6B4B,$6C24,$6CF9,$6DCA,$6E97,$6F5F,$7023
	dc.w	$70E3,$719E,$7255,$7308,$73B6,$7460,$7505,$75A6
	dc.w	$7642,$76D9,$776C,$77FB,$7885,$790A,$798A,$7A06
	dc.w	$7A7D,$7AEF,$7B5D,$7BC6,$7C2A,$7C89,$7CE4,$7D3A
	dc.w	$7D8A,$7DD6,$7E1E,$7E60,$7E9D,$7ED6,$7F0A,$7F38
	dc.w	$7F62,$7F87,$7FA7,$7FC2,$7FD9,$7FEA,$7FF6,$7FFE
	dc.w	$7FFF


*************************************************************************************
*
* objects used

wasp_objects					* wasp objects
x_pos		EQU	*-wasp_objects		* the x co-ordinate
	ds.w	1
y_pos		EQU	*-wasp_objects		* the wasp y co-ordinate
	ds.w	1
z_pos		EQU	*-wasp_objects		* the wasp z co-ordinate
	ds.w	1
x_vel		EQU	*-wasp_objects		* the wasp x velocity
	ds.w	1
y_vel		EQU	*-wasp_objects		* the wasp y velocity
	ds.w	1
z_vel		EQU	*-wasp_objects		* the wasp z velocity
	ds.w	1
x_old		EQU	*-wasp_objects		* the previous wasp x co-ordinate
	ds.w	1
y_old		EQU	*-wasp_objects		* the previous wasp y co-ordinate
	ds.w	1
z_old		EQU	*-wasp_objects		* the previous wasp z co-ordinate
	ds.w	1

wasp_size	EQU	*-wasp_objects		* size of one wasp object
	ds.w	(number_of_wasps-1)*wasp_size	* space for the other wasps

bee_objects						* bee objects
	ds.w	1					* the bee x co-ordinate
	ds.w	1					* the bee y co-ordinate
	ds.w	1					* the bee z co-ordinate
	ds.w	1					* the bee x velocity
	ds.w	1					* the bee y velocity
	ds.w	1					* the bee z velocity
	ds.w	1					* the previous bee x co-ordinate
	ds.w	1					* the previous bee y co-ordinate
	ds.w	1					* the previous bee z co-ordinate

bee_size	EQU	*-bee_objects		* size of one bee object
	ds.w	(number_of_bees-1)*bee_size	* space for the other bees


*************************************************************************************
*
* variables used

variables						* points to variables base

	OFFSET	0				* going to use relative addressing

PRNlword	ds.l	1				* PRNG seed long word

yaw		ds.w	1				* environment rotation about y axis

ebox		ds.w	1				* environment origin x
eboy		ds.w	1				* environment origin y
eboz		ds.w	1				* environment origin z

x1		ds.w	1				* x co-ordinate
y1		ds.w	1				* y co-ordinate
x2		ds.w	1				* x co-ordinate
y2		ds.w	1				* y co-ordinate
x3		ds.w	1				* x co-ordinate
y3		ds.w	1				* y co-ordinate
x4		ds.w	1				* x co-ordinate
y4		ds.w	1				* y co-ordinate
x5		ds.w	1				* x co-ordinate
y5		ds.w	1				* y co-ordinate
x6		ds.w	1				* x co-ordinate
y6		ds.w	1				* y co-ordinate
x7		ds.w	1				* x co-ordinate
y7		ds.w	1				* y co-ordinate
x8		ds.w	1				* x co-ordinate
y8		ds.w	1				* y co-ordinate

status
	ds.w	1					* show status flag
key_status
	ds.w	1					* status key last state
key_wupdnltrt
	ds.l	1					* wasp key last state
key_bupdnltrt
	ds.l	1					* bee key last state

screen_width
	ds.w	1					* screen width
screen_height
	ds.w	1					* screen height

wasp_acceleration
	ds.w	1					* how fast the wasps can accelerate
bee_acceleration
	ds.w	1					* how fast the bees can accelerate

wasp_velocity_max
	ds.w	1					* maximum wasp (x^2 + y^2) velocity
bee_velocity_max
	ds.w	1					* maximum bee (x^2 + y^2) velocity


	END	start


*************************************************************************************
